@@ -227,6 +227,8 @@ impl Inner {
 
             self.handlers.close.call_simple();
 
+            let subscription_handlers: Vec<_> = mem::take(&mut self.subscription_handlers.lock());
+
             if close_request {
                 let channel = self.channel.clone();
                 let request = DataConsumerCloseRequest {
@@ -237,8 +239,6 @@ impl Inner {
                         data_producer_id: self.data_producer_id,
                     },
                 };
-                let subscription_handlers: Vec<_> =
-                    mem::take(&mut self.subscription_handlers.lock());
                 let weak_data_producer = self.weak_data_producer.clone();
 
                 self.executor
@@ -254,6 +254,14 @@ impl Inner {
                         drop(subscription_handlers);
                     })
                     .detach();
+            } else {
+                self.executor
+                    .spawn(async move {
+                        // Drop from a different thread to avoid deadlock with recursive dropping
+                        // from within another subscription drop.
+                        drop(subscription_handlers);
+                    })
+                    .detach();
             }
         }
     }
